/********************************************************************
noiseclock: sumarize time of mp3 and other sound formats
Copyright (C) 2004 Jack Chatterton <chattj AT postreal.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

*********************************************************************/

/* Like ftw, this code walks a file tree and will run a particular 
 routine (fn *) on all directories. Unlike ftw, it will not run the
 function on plain files, which should make it faster for this application.
 Also unlike ftw, it does not bound the number of open directories. 
*/

#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include "utilities.h"

#define MAX_FILENAME_SIZE 2048

/* Recursively walks down the directories. */
int rec_dirwalk(char *target, int (*fn) ()) {
    DIR *dp;
    struct dirent *ep;
    int returnme;
	char *curdir;

	/* Ugh, so expensive. But for this to be really dynamic this has
	 to be here, right? */
	curdir = safe_malloc(MAX_FILENAME_SIZE);

    dp = opendir(target);
    if(dp != NULL) {
		/* Here is the base case for the recursion. Even if there are no 
		 subdirectories to (target), the fn will be called on it and the
		 fn's return value returned. */
        returnme = (*fn) (target); 
        /* Compiler wants me to put weird parens here */
        while((ep = readdir(dp))) {
			/* Unfortunately, readdir returns the dot directories */
            if((ep->d_type == DT_DIR) && (strcmp(ep->d_name, ".") != 0) && (strcmp(ep->d_name, "..") != 0)) {
				strcpy(curdir, target);
				strcat(curdir, "/");
				strcat(curdir, ep->d_name);
                rec_dirwalk(curdir, (*fn));
            }
            /* else do nothing */
        }
		closedir(dp);
    } else {
        return 0;
    }
	safe_free(curdir);
    return returnme;
}

/* Here is the externally available function to call */
int dirwalk(char *target, int (*fn) ()) {
    char *fixedtarget;
    int length, returnme;
    fixedtarget = safe_malloc(MAX_FILENAME_SIZE);
    length = strlen(target);
    if(target[length-1] == '/') {
        /* I don't understand why the below code is necessary, since it seems
        like one should just be able to strncpy(fixed, target, length - 1) 
        and get the same results. Still, this is error-free now. */
        strcpy(fixedtarget, "");
        strncat(fixedtarget, target, length - 1);
    } else {    
        strcpy(fixedtarget, target);
    }
    returnme = rec_dirwalk(fixedtarget, fn);
    safe_free(fixedtarget);
    return returnme;
}
