/********************************************************************

noiseclock: sumarize time of mp3 and other sound formats
Copyright (C) 2004 Jack Chatterton <chattj AT postreal.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
02111-1307, USA.

*********************************************************************/

/* Set up a given format. This is a singleton module -- you can't have
 more than one formatter at once. Formatstr has the options of including:
 d - days, h - hours, m - minutes, s - seconds. Order is important, and 
 repeats are allowed, though whitespace is ignored. */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define SECS_IN_MINUTE 60
#define SECS_IN_HOUR 3600
#define SECS_IN_DAY 86400

char *formatstr = NULL;
char *returnme = NULL;
/* Array to signify which units are in play. Set these to nonzero
for units which are to be used. Ordering is most significant first,
i.e. [d][h][m][s] */
int significant_units[4];

/* Build the formatter. Return -1 if the string won't work. */
int setup_formatter(char *fstr) {
	int size, i; 
	int strlength = 0;
	/* Zero out significant units. */ 
	significant_units[0] = 0;
	significant_units[1] = 0;
	significant_units[2] = 0;
	significant_units[3] = 0;
	/* make sure format string is only valid characters */
	formatstr = (char *) malloc(sizeof(fstr));
	strcpy(formatstr, fstr);
	size = strlen(formatstr);
	if(!(strspn(formatstr, "dhms") == size)) 
		return -1;
	/* create significant units array */
	for(i = 0; i < size; i++) {
		switch(formatstr[i]) {
			case 'd':
				significant_units[0] = 1;
				strlength += 6; /* 4 digits + 1 unit + spc */
				break;
			case 'h':
				significant_units[1] = 1;
				strlength += 4; /* 2 digits + 1 unit + spc  */
				break;
			case 'm':
				significant_units[2] = 1;
				strlength += 4; /* 2 digits + 1 unit + spc  */
				break;
			case 's':
				significant_units[3] = 1;
				strlength += 10; /* 8 digits + 1 unit + spc  */
				break;
		}
	}
	/* returnme will be 5 times the length of the format string, because
	 in no case should each formatted unit string be more than 5 
	 characters long, counting spaces. HACK. */
	size = strlength * sizeof(char) * 5;
	returnme = (char *) malloc(size);
	return 1;
}

/* Return formatted time, according to formatstr. Set human to zero
 to make this not mark units (ie not human readable). */
char *time_format(int secs, int human) {
	int days = 0, hours = 0, minutes = 0, seconds = secs; 
	int size, i;
	/* These three clauses process the time from seconds to other units
	 as necessary. */
	if(significant_units[0]) {
		days = (int) (seconds / SECS_IN_DAY);
		seconds = seconds % SECS_IN_DAY;
	}
	if(significant_units[1]) {
		hours = (int) (seconds / SECS_IN_HOUR);
		seconds = seconds % SECS_IN_HOUR;
	}
	if(significant_units[2]) {
		minutes = (int) (seconds / SECS_IN_MINUTE);
		seconds = seconds % SECS_IN_MINUTE;
	}
	strcpy(returnme , "");
	size = strlen(formatstr);
	/* This part prints the numbers to the string which will be used
	 for output. */
	for(i = 0; i < size; i++) {
		switch(formatstr[i]) {
			case 'd':
				sprintf(returnme, "%s%2i", returnme, days);
				if(human)
					strcat(returnme, "d");
				strcat(returnme, " ");
				break;
			case 'h':
				sprintf(returnme, "%s%3i", returnme, hours);
				if(human)
					strcat(returnme, "h");
				strcat(returnme, " ");
				break;
			case 'm':
				sprintf(returnme, "%s%2i", returnme, minutes);
				if(human)
					strcat(returnme, "m");
				strcat(returnme, " ");
				break;
			case 's':
				sprintf(returnme, "%s%2i", returnme, seconds);
				if(human)
					strcat(returnme, "s");
				strcat(returnme, " ");
				break;
		}
	}
	return returnme;
}
