/********************************************************************

noiseclock: sumarize time of mp3 and other sound formats
Copyright (C) 2004 Jack Chatterton <chattj AT postreal.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
02111-1307, USA.

*********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*************** Turn stderr on and off **************/

/* 2 is the posix place for stderr to point */
int stderr_fileno = 2;
int is_suppressed = 0;

/* Disallow errors, by re-routing stderr. Useful when importing
 noisy libraries. */
void suppress_errors() {
	if(is_suppressed == 1)
		return;
	is_suppressed = 1;
	stderr_fileno = fileno(stderr);
	if(!freopen("/dev/null", "w", stderr)) 
		printf("Could not suppress error messages!\n");
	return;
}

/* Re-allow errors */
void allow_errors() {
	if(is_suppressed == 0)
		return;
	is_suppressed = 0;
	if(!fdopen(stderr_fileno, "w"))
		printf("Could not re-allow error messages after suppression!\n");
	return;
}

/*************** Safe memory allocation **************/

/* Show whether or not errors are allowed */
int errors_allowed() {
	if(is_suppressed) 
		return 0;
	else 
		return 1;
}

void *safe_malloc(int size) {
    void *ptr;
    ptr = NULL;
    ptr = malloc(size);
    if(!ptr) {
        printf("ERROR: Out of memory. Exiting.\n");
        exit(-1);
    }
    return ptr;
}

void safe_free(void *ptr) {
    free(ptr);
    ptr = NULL;
}

/* Counts the number of slashes in a string. */
int countslashes(char *input) {
	int i;
	int total = 0;
	for(i = 0; i < strlen(input); i++) 
		if(input[i] == '/')
			total++;
	return total;
}
